Package: ETDQualitizer
Type: Package
Title: Automated Eye Tracking Data Quality Determination for
        Screen-Based Eye Trackers
Version: 1.1.0
Authors@R: c(
    person(
      "Diederick", "Niehorster",
      email = "diederick_c.niehorster@humlab.lu.se",
      role = c("aut", "cre", "cph"),
      comment = c(ORCID = "0000-0002-4672-8756")
    )
  )
Maintainer: Diederick Niehorster <diederick_c.niehorster@humlab.lu.se>
Description: Compute common data quality metrics for accuracy, precision and data loss
  for screen-based eye trackers. The package supports gaze input in screen pixels or
  degrees and reports angular measures in degrees where appropriate. If you use this
  package, please cite Niehorster, D.C., Nyström, M., Hessels, R.S., Benjamins, J.S.,
  Andersson, R., and Hooge, I.T.C. (2026). The fundamentals of eye tracking, Part 7:
  Determining data quality. Behavior Research Methods. <doi:10.3758/s13428-026-03039-4>.
License: MIT + file LICENSE
URL: https://github.com/dcnieho/ETDQualitizer
BugReports: https://github.com/dcnieho/ETDQualitizer/issues
Encoding: UTF-8
Imports: R6, stats
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-06-07 22:17:16 UTC; runner
Author: Diederick Niehorster [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-4672-8756>)
Repository: CRAN
Date/Publication: 2026-06-08 06:50:19 UTC
Built: R 4.5.2; ; 2026-06-08 08:14:12 UTC; unix

Package: ETDQualitizer
Type: Package
Title: Automated Eye Tracking Data Quality Determination for
        Screen-Based Eye Trackers
Version: 1.1.0
Authors@R: c(
    person(
      "Diederick", "Niehorster",
      email = "diederick_c.niehorster@humlab.lu.se",
      role = c("aut", "cre", "cph"),
      comment = c(ORCID = "0000-0002-4672-8756")
    )
  )
Maintainer: Diederick Niehorster <diederick_c.niehorster@humlab.lu.se>
Description: Compute common data quality metrics for accuracy, precision and data loss
  for screen-based eye trackers. The package supports gaze input in screen pixels or
  degrees and reports angular measures in degrees where appropriate. If you use this
  package, please cite Niehorster, D.C., Nyström, M., Hessels, R.S., Benjamins, J.S.,
  Andersson, R., and Hooge, I.T.C. (2026). The fundamentals of eye tracking, Part 7:
  Determining data quality. Behavior Research Methods. <doi:10.3758/s13428-026-03039-4>.
License: MIT + file LICENSE
URL: https://github.com/dcnieho/ETDQualitizer
BugReports: https://github.com/dcnieho/ETDQualitizer/issues
Encoding: UTF-8
Imports: R6, stats
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-06-07 22:17:16 UTC; runner
Author: Diederick Niehorster [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-4672-8756>)
Repository: CRAN
Date/Publication: 2026-06-08 06:50:19 UTC
Built: R 4.5.2; ; 2026-06-08 10:01:26 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DataQuality)
export(ETDQ_version)
export(Fick_to_vector)
export(ScreenConfiguration)
export(accuracy)
export(bcea)
export(compute_data_quality_from_validation)
export(data_loss_from_expected)
export(data_loss_from_invalid)
export(effective_frequency)
export(precision_using_moving_window)
export(report_data_quality_table)
export(rms_s2s)
export(std)
export(vector_to_Fick)
importFrom(R6,R6Class)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,median)
importFrom(stats,sd)

[![PyPI Latest Release](https://img.shields.io/pypi/v/ETDQualitizer.svg)](https://pypi.org/project/ETDQualitizer/)
[![Python Downloads](https://static.pepy.tech/badge/ETDQualitizer)](https://pepy.tech/project/ETDQualitizer)
[![File Exchange](https://www.mathworks.com/matlabcentral/images/matlab-file-exchange.svg)](https://se.mathworks.com/matlabcentral/fileexchange/181899-etdqualitizer)
[![CRAN](https://www.r-pkg.org/badges/version/ETDQualitizer?color=green)](https://cran.r-project.org/package=ETDQualitizer)
[![](http://cranlogs.r-pkg.org/badges/grand-total/ETDQualitizer?color=green)](https://cran.r-project.org/package=ETDQualitizer)

# ETDQualitizer v0.10.1
ETDQualitizer is a toolbox for automated eye tracking data quality determination for screen-based eye trackers. This repository consists of two parts:
1. The Validator module: A [Python module for collecting validation data](#collecting-validation-data) in a format that is suitable for automated determination and reporting of data quality for screen-based eye trackers. This module can both be run as a standalone Python script using [PsychoPy](https://www.psychopy.org/) or an integratable module that can be added to an exisiting PsychoPy experiment using a single line of code.
2. ETDQualitizer: a [tool for determining and reporting eye-tracking data quality](#determining-data-quality) using recordings made by the Validator script, or any data formatted according to the same format. ETDQualitizer is available for MATLAB, Python and R, and can furthermore be run [as a webpage](https://dcnieho.github.io/ETDQualitizer/).

Note that for determining data quality for recordings made with wearable eye trackers, the [glassesValidator](https://github.com/dcnieho/glassesValidator) tool is available (which is also integrated in [gazeMapper](https://github.com/dcnieho/gazeMapper)).

Please cite:
Niehorster, D.C., Nyström, M., Hessels, R.S., Benjamins, J.S., Andersson, R. & Hooge, I.T.C. (in prep). The fundamentals of eye tracking part 7: Determining data quality

For questions, bug reports or to check for updates, please visit
www.github.com/dcnieho/ETDQualitizer.

# Content
Below, the following topics are discussed:
1. [The Validator module, its configuration, and its output data format](#collecting-validation-data).
2. [The ETDQualitizer tool for determining and reporting eye-tracking data quality](#determining-data-quality).
3. [A complete walkthrough using the Validator script and ETDQualitizer](#walkthrough).

# Collecting validation data
The procedure for collecting validation data is available from [the `/python/ETDQualitizer/validator` subfolder](/python/ETDQualitizer/validator). In this manual, we will refer to this script as the validator.

## How to acquire
The validator is not available as an installable package. The user is recommended to download [the `/python/ETDQualitizer/validator` subfolder](/python/ETDQualitizer/validator) from github and add it to their experiment.

## Documentation
Several files are contained in [the `/python/ETDQualitizer/validator` subfolder](/python/ETDQualitizer/validator) that together comprise the validator. The validator can be run in two ways, either as a standalone script, or integrated into an existing PsychoPy script. By default, support is included for EyeLink, SMI and Tobii eye trackers. To run as a standalone script, simply run the `run_for_eyelink.py`, `run_for_smi.py`, or `run_for_tobii.py` scripts. To integrate the validator into an existing script, simply call the `run_validation()` function inside the `validator.py` module in your existing PsychoPy script at appropriate time(s).

## Configuration files
The procedure (e.g., position of the validation targets, and duration for which they are shown) is configured using the following two configuration files.

### `targetPositions.csv`
The `targetPositions.csv` file contains the positions (and associated ID) for the validation targets. It contains the following columns:
|name|description|
| --- | --- |
| `ID` | Unique identifier of the validation target (integer). |
| `x` | Horizontal position of the center of the validation target. |
| `y` | Vertical position of the center of the validation target. |
| `color` | Optional column allowing to set the color of the target place holder for this target. |

Validation target coordinates are in PsychoPy's `'pix'` coordinate system. That is, they are in pixels with respect to the center of the screen, with the positive y direction being upward.

### `setup.json`
The `setup.json` file contains information about the screen and configuration for the validation procedure. The below table describes the options. It is a nested json file; nesting levels are indicated with `.` in the table.
|name|description|
| --- | --- |
| `screen.which_monitor` | Which monitor the validation screen will be shown on (only used for standalone operation) |
| `screen.width` | The width of the monitor (its display area to be exact) in cm |
| `screen.viewing_distance` | The distance from which the monitor is viewed |
| `screen.refresh_rate` | The refresh rate of the monitor |
| `screen.resolution` | The pixel resolution of the monitor |
| `screen.background_color` | The background color to use for the validation display |
|||
| `instruction_text.height` | The size of lines of the instruction text |
| `instruction_text.color` | The color of the instruction text |
|||
| `validation.n_repetitions` | The number of times each validation target should be shown during a single validation session |
| `validation.targets.file` | The name of the file containing the validation targets, probably [`targetPositions.csv`](#targetpositionscsv) |
| `validation.targets.duration` | The duration of gaze data to capture for each target |
| `validation.targets.randomize_order` | Boolean indicating whether the target presentation order should be shuffled or as in the file |
| `validation.targets.placeholder.color` | Color of the placeholder to be shown at all fixation target locations (`null`, Python's `None` to disable) |
| `validation.targets.placeholder.diameter` | Diameter of the placeholder circle |
| `validation.targets.cue.color` | Color of the cue to be shown as the fixation marker is traveling to a next fixation target location (`null`, Python's `None` to disable) |
| `validation.targets.cue.diameter` | Diameter of the cue circle |
| `validation.targets.move.duration` | Duration it would take for the fixation target to move from one end of the screen to the other, horizontally. This duration is scaled by the actual distance between the current and the next fixation target location |
| `validation.targets.move.diameter` | Diameter of the fixation target as it is moving |
| `validation.targets.move.min_duration` | Minimum duration of move from the current to next fixation target location |
| `validation.targets.move.move_with_acceleration` | Boolean indicating whether the target should accelerate and decelerate as it moves between the current and next fixation target locations, or move at a constant speed |
| `validation.targets.shrink.duration` | Duration of interval during which the target shrinks after it has arrived at the locaiton of the upcoming fixation target |
| `validation.targets.look.diameter_min` | Diameter at the start of the shrinking interval |
| `validation.targets.look.diameter_max` | Diameter at the end of the shrinking interval|
| `validation.targets.look.outer_color` | Color of the large and small circles in the fixation target |
| `validation.targets.look.inner_color` | Color of the cross in the fixation target |

All sizes are in pixels, and all durations in seconds. Ensure that the screen information is correctly filled out or the timing or positioning of validation targets may be incorrect.

## Output data format
The data files output by the validator contain raw gaze data (indvidual samples) with the following columns:
|name|description|
| --- | --- |
| `timestamp` | Timestamp of the sample, in ms |
| `left_x` | Horizontal position of the gaze point of the left eye, in pixels |
| `left_y` | Vertical position of the gaze point of the left eye, in pixels |
| `right_x` | Horizontal position of the gaze point of the right eye, in pixels |
| `right_y` | Vertical position of the gaze point of the right eye, in pixels |
| `target_id` | ID of the target at which the participant looked. -1 for samples that should not be included in the data quality calculations, an integer ID (see description of [`targetPositions.csv`](#targetpositionscsv)) for intervals to be included |
| `tar_x` | Horizontal screen position of the center of the target, in pixels |
| `tar_y` | Vertical screen position of the center of the target, in pixels |

Note that positions in this file _do not_ use PsychoPy's `'pix'` coordinate system where the positive y direction is upward. Like in most other conventions, the positive y direction is downward. Positions of both gaze and targets are expressed with respect to the center of the screen. Note that it is possible a file contains data from only the left or only the right eye, in which case the other columns should be missing.

## Adding your own eye tracker
The validator provides out of the box support for EyeLink, SMI and Tobii eye trackers. To add support for other eye trackers, users should add a class wrapping communication with their eye tracker in [`validator.py`](/python/ETDQualitizer/validator/validator.py). This class should derive from the `EyeTrackerBase` class in that file, and implement the methods `start_recording`, `send_message`, `stop_recording`, and `save_data`. See the `EyeLinkTracker`, `SMIEyeTracker`, and `TobiiEyeTracker` classes for examples.

# Determining Data Quality
Example scripts for using the MATLAB, Python and R implementations, as well as example data are available from [the `/example` subfolder](/example).

## How to acquire
While the [MATLAB](/matlab), [Python](/python/ETDQualitizer) and [R](/r/ETDQualitizer) implementations can be directly downloaded from their respective subfolders in this repository, the recommended installation method is to use the respective platform's package manager:
### MATLAB
Search for and install the ETDQualitizer package from the MATLAB Add-Ons manager.
### Python
Execute `pip install ETDQualitizer`
### R
Execute `install.packages("ETDQualitizer")`

## Documentation
The documentation is integrated with the code and accessible using the `help` function in MATLAB, Python and R. As such, it will not be repeated here. Here we provide a high-level overview of the ETDQualitizer package's design.
- Basic functions: the package provides the following functions used for determining data quality of a segment of eye tracking data: `accuracy`, `rms_s2s`, `std`, `bcea`, `data_loss_from_invalid`, `data_loss_from_expected`, `effective_frequency`. Furthermore, the function `precision_using_moving_window` is provided for determining RMS-S2S, STD or BCEA using a moving window approach. The algorithm implemented in each of these functions is described in Niehorster et al. (in prep).
- Coordinate transformation class and functions: the package provides the `ScreenConfiguration` class, which can be used to convert data represented as pixels on a monitor to physical distances in the world (e.g., mm) and to angular gaze directions (using Fick angles), and vice-versa. Further available are the functions `Fick_to_vector` and `vector_to_Fick` that turn angular gaze directions into gaze vectors, and vice versa.
- Data quality computation class: the package provides the `DataQuality` class, which is a wrapper around the basic functions above. It is constructed using a segment of eye tracking data (optionally converted to gaze angles if they are not already expressed as such). The various data quality measures can then be directly computed on the segment by using the class methods.
- Wrapper functions: finally, the package provides two higher-level wrapper functions, `compute_data_quality_from_validation` and `report_data_quality_table`. `compute_data_quality_from_validation` can be used to take a segment of eye tracking formatted according to [the format produced by the Validator module](#output-data-format) and calculate the various data quality metrics for each presented target in this segment. The `report_data_quality_table` then takes one or multiple tables output by `compute_data_quality_from_validation` and provides an overall summary of data quality across targets and (if multiple tables are provided) recordings/participants, along with a textual report that can be directly pasted into a paper.

# Walkthrough
This walkthrough will help you become familiar with how to use the Validator script and the ETDQualitizer package to determine the data quality of an eye tracking recording. It uses the [standalone Validator script for a Tobii] as an example, along with the [ETDQualitizer tool packaged as a webpage](https://dcnieho.github.io/ETDQualitizer/). Users who prefer to use ETDQualitizer from their own MATLAB, Python or R script are referred to the [respective example scripts](/example). The walkthrough is organized in two parts, 1) acquiring gaze data that is suitable for determining data quality and 2) determining eye tracking data quality from this gaze data using ETDQualitizer. If one only wishes to learn how to determine data quality using ETDQualitizer, one can skip to this section and use the provided [example data](/example/data).

## Collecting gaze data for data quality assessment using the Validator script

## Determining data quality for screen-based eye tracker recordings using ETDQualitizer

# Citation
If you use this tool or any of the code in this repository, please cite:<br>
Niehorster, D.C., Nyström, M., Hessels, R.S., Benjamins, J.S., Andersson, R. & Hooge, I.T.C. (in prep). The fundamentals of eye tracking part 7: Determining data quality

## BibTeX
```latex
@article{niehorsterFundamentals7,
    Author = {Niehorster, Diederick C. and
              Nystr{\"o}m, Marcus and
              Hessels, Roy S. and
              Benjamins, Jeroen S. and
              Andersson, Richard and
              Hooge, Ignace T. C.},
    Journal = {},
    Number = {},
    Title = {The fundamentals of eye tracking part 7: Determining data quality},
    Year = {in prep},
    doi = {}
}
```

# Developer information
## Release process
1. First make a new tag and push it. This will trigger the on_tag_prep_release.yml workflow, which will build the needed Matlab FEX and CRAN packages and then prepare a release.
2. Edit the prepared release and finalize it. This will trigger upload to PyPI and Matlab FEX. CRAN uploads have to be done manually.

library(R6)
library(stats)


#' Get ETDQualitizer Version
#'
#' Returns the current version string of the ETDQualitizer tool.
#'
#' @return A character string representing the version number.
#'
#' @examples
#' ETDQ_version()
#'
#' @export
ETDQ_version <- function() {
  return("1.0.0")
}


#' Convert Fick Angles to 3D Vector
#'
#' Converts azimuth and elevation angles (in degrees) to a 3D unit vector.
#'
#' @param azi Azimuth angle in degrees.
#' @param ele Elevation angle in degrees.
#' @param rho Radius (default is 1.0).
#'
#' @return A list with components \code{x}, \code{y}, and \code{z}.
#' @examples
#' Fick_to_vector(30, 10)
#' @export
Fick_to_vector <- function(azi, ele, rho=1.0) {
  # Convert degrees to radians
  azi_rad <- pi/180 * azi
  ele_rad <- pi/180 * ele
  r_cos_ele <- rho * cos(ele_rad)
  x <- r_cos_ele * sin(azi_rad)
  y <-       rho * sin(ele_rad)
  z <- r_cos_ele * cos(azi_rad)
  list(x = x, y = y, z = z)
}

#' Convert 3D Vector to Fick Angles
#'
#' Converts a 3D vector to azimuth and elevation angles (in degrees).
#'
#' @param x X component of the vector.
#' @param y Y component of the vector.
#' @param z Z component of the vector.
#'
#' @return A list with components \code{azi} and \code{ele}.
#' @examples
#' vector_to_Fick(0.5, 0.2, 0.8)
#' @export
vector_to_Fick <- function(x, y, z) {
  azi <- 180/pi * atan2(x, z)
  ele <- 180/pi * atan2(y, sqrt(x^2 + z^2))
  list(azi = azi, ele = ele)
}

call_summary_fun <- function(fun, x) {
  fun_formals <- formals(fun)
  if (!is.null(fun_formals) && ("na.rm" %in% names(fun_formals) || "..." %in% names(fun_formals))) {
    return(fun(x, na.rm = TRUE))
  }

  fun(x[!is.na(x)])
}


#' Compute Gaze Accuracy
#'
#' Calculates the angular offset between gaze and target directions by first
#' computing a central gaze direction in 3D and then expressing this direction
#' relative to the target.
#'
#' @param azi Gaze azimuth in degrees.
#' @param ele Gaze elevation in degrees.
#' @param target_azi Target azimuth in degrees.
#' @param target_ele Target elevation in degrees.
#' @param central_tendency_fun Function to compute central tendency of the Cartesian gaze components (default: \code{mean}).
#'
#' @return A list with \code{offset}, \code{offset_azi}, and \code{offset_ele}, the total, horizontal and vertical offset of gaze from the target (in degrees).
#' @examples
#' accuracy(c(1, 2), c(1, 2), 0, 0)
#' @export
accuracy <- function(azi, ele, target_azi, target_ele, central_tendency_fun = mean) {
  # convert gaze directions to unit vectors
  g <- Fick_to_vector(azi, ele)

  # compute central gaze direction in 3D
  g_vec <- c(
    call_summary_fun(central_tendency_fun, g$x),
    call_summary_fun(central_tendency_fun, g$y),
    call_summary_fun(central_tendency_fun, g$z)
  )

  # normalize to unit vector
  g_vec <- g_vec / sqrt(sum(g_vec^2))

  # precompute trigonometric terms for target orientation
  ca <- cos(target_azi * pi/180); sa <- sin(target_azi * pi/180)
  ce <- cos(target_ele * pi/180); se <- sin(target_ele * pi/180)

  # express central gaze direction in a target-centered frame
  x_rel <-  ca*g_vec[1] - sa*g_vec[3]
  y_rel <-  ce*g_vec[2] - se*(sa*g_vec[1] + ca*g_vec[3])
  z_rel <-  se*g_vec[2] + ce*(sa*g_vec[1] + ca*g_vec[3])

  # decompose relative direction into Fick components
  f <- vector_to_Fick(x_rel, y_rel, z_rel)
  offset_azi <- f$azi
  offset_ele <- f$ele

  # compute total angular offset
  offset <- atan2(sqrt(x_rel^2 + y_rel^2), z_rel) * 180/pi

  list(offset = offset, offset_azi = offset_azi, offset_ele = offset_ele)
}

#' RMS of Sample-to-Sample Differences
#'
#' Computes root mean square of differences between successive gaze samples.
#'
#' @param azi Azimuth values in degrees.
#' @param ele Elevation values in degrees.
#' @param central_tendency_fun Function to compute central tendency (default: \code{mean}).
#'
#' @return A list with \code{rms}, \code{rms_azi}, and \code{rms_ele}, the total RMS of sample-to-sample distances and that of the azimuthal and elevation components (all in degrees).
#' @examples
#' rms_s2s(c(1, 2, 3), c(1, 2, 3))
#' @export
rms_s2s <- function(azi, ele, central_tendency_fun = mean) {
  a_diff  <- diff(azi)^2
  e_diff  <- diff(ele)^2
  rms_azi <- sqrt(call_summary_fun(central_tendency_fun, a_diff))
  rms_ele <- sqrt(call_summary_fun(central_tendency_fun, e_diff))
  rms     <- sqrt(call_summary_fun(central_tendency_fun, a_diff + e_diff))
  list(rms = rms, rms_azi = rms_azi, rms_ele = rms_ele)
}

#' Standard Deviation of Gaze Samples
#'
#' Computes standard deviation of azimuth and elevation.
#'
#' @param azi Azimuth values in degrees.
#' @param ele Elevation values in degrees.
#'
#' @return A list with \code{std}, \code{std_azi}, and \code{std_ele}, the total STD of sample-to-sample distances and that of the azimuthal and elevation components (all in degrees).
#' @examples
#' std(c(1, 2, 3), c(1, 2, 3))
#' @export
std <- function(azi, ele) {
  std_azi <- pop_sd(azi)
  std_ele <- pop_sd(ele)
  std     <- sqrt(std_azi^2 + std_ele^2)
  list(std = std, std_azi = std_azi, std_ele = std_ele)
}
#' Population Standard Deviation
#'
#' Computes population standard deviation of a numeric vector.
#'
#' @param x Numeric vector.
#'
#' @return Population standard deviation.
#' @examples
#' pop_sd(c(1, 2, 3, 4))
#' @noRd
pop_sd <- function(x) {
  x <- x[!is.na(x)]
  sqrt(mean((x - mean(x))^2))
}

#' Bivariate Contour Ellipse Area (BCEA)
#'
#' Computes BCEA and ellipse parameters for gaze precision.
#'
#' @param azi Azimuth values in degrees.
#' @param ele Elevation values in degrees.
#' @param P Cumulative probability (default: 0.68).
#'
#' @return A list with the BCEA (\code{area}) and additional info about the BCEA ellipse: \code{orientation}, \code{ax1}, \code{ax2}, and \code{aspect_ratio}.
#' @examples
#' bcea(rnorm(100), rnorm(100))
#' @export
#' @importFrom stats cor cov sd
bcea <- function(azi, ele, P = 0.68) {
  # turn cumulative probability into scale factor
  k <- log(1 / (1 - P))
  # remove NaNs
  valid <- !(is.na(azi) | is.na(ele))
  azi_valid <- azi[valid]
  ele_valid <- ele[valid]
  # compute sample standard deviations
  std_a <- sd(azi_valid)
  std_e <- sd(ele_valid)
  # compute correlation
  rho <- cor(azi_valid, ele_valid)
  # compute BCEA
  area <- 2 * k * pi * std_a * std_e * sqrt(1 - rho^2)

  # compute major and minor axis radii, and orientation, of the BCEA ellipse
  cov_matrix <- cov(cbind(azi_valid, ele_valid))
  eig <- eigen(cov_matrix)
  i <- which.max(eig$values)
  orientation <- atan2(eig$vectors[2, i], eig$vectors[1, i]) * 180 / pi
  ax1 <- sqrt(k * eig$values[ i])
  ax2 <- sqrt(k * eig$values[-i])
  aspect_ratio <- max(ax1, ax2) / min(ax1, ax2)
  # sanity check: this (formula for area of ellipse) should
  # closely match directly computed area from above
  # 2*pi*ax1*ax2
  list(
    area = area,
    orientation = orientation,
    ax1 = ax1,
    ax2 = ax2,
    aspect_ratio = aspect_ratio
  )
}

#' Compute Data Loss from number of invalid samples.
#'
#' Calculates percentage of missing gaze samples.
#'
#' @param a Horizontal gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).
#' @param b Vertical gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).
#'
#' @return Percentage of missing samples.
#' @examples
#' data_loss_from_invalid(c(1, NA, 3), c(1, 2, NA))
#' @export
data_loss_from_invalid <- function(a, b) {
  missing <- is.na(a) | is.na(b)
  sum(missing)/length(missing)*100
}

#' Compute Data Loss from Expected Sample Count
#'
#' Calculates data loss based on expected number of samples.
#'
#' @param a Horizontal gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).
#' @param b Vertical gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).
#' @param duration Duration in seconds.
#' @param frequency Sampling frequency in Hz.
#'
#' @return Percentage of data loss.
#' @examples
#' data_loss_from_expected(c(1, NA, 3), c(1, 2, NA), duration = 1, frequency = 3)
#' @export
data_loss_from_expected <- function(a, b, duration, frequency) {
  N_valid <- sum(!is.na(a) & !is.na(b))
  (1 - N_valid/(duration* frequency))*100
}

#' Compute Effective Sampling Frequency
#'
#' Calculates effective frequency based on valid samples.
#'
#' @param a Horizontal gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).
#' @param b Vertical gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).
#' @param duration Duration in seconds.
#'
#' @return Effective frequency in Hz.
#' @examples
#' effective_frequency(c(1, NA, 3), c(1, 2, NA), duration = 1)
#' @export
effective_frequency <- function(a, b, duration) {
  N_valid <- sum(!is.na(a) & !is.na(b))
  N_valid/duration
}


#' Precision Using Moving Window
#'
#' Computes gaze precision using a moving window and selected metric.
#'
#' @param azi Azimuth values.
#' @param ele Elevation values.
#' @param window_length Window size in samples.
#' @param metric Precision metric: \code{"RMS-S2S"}, \code{"STD"}, or \code{"BCEA"}.
#' @param aggregation_fun Function to aggregate precision values across the windows (default: \code{median}).
#' @param ... Additional arguments passed to metric function.
#'
#' @return Aggregated precision value.
#' @examples
#' precision_using_moving_window(rnorm(100), rnorm(100), 10, "STD")
#' @export
#' @importFrom stats median
precision_using_moving_window <- function(azi, ele, window_length, metric, aggregation_fun = median, ...) {
  # Select the appropriate precision metric function
  fun <- switch(metric,
    "RMS-S2S" = rms_s2s,
    "STD"     = std,
    "BCEA"    = bcea,
    stop(sprintf('metric "%s" is not understood', metric))
  )

  # Get number of samples in data
  ns <- length(azi)

  if (window_length < ns) {
    # If number of samples in data exceeds window size
    values <- rep(NA_real_, ns - window_length + 1)  # pre-allocate
    for (p in seq_len(ns - window_length + 1)) {
      result <- fun(azi[p:(p + window_length - 1)], ele[p:(p + window_length - 1)], ...)
      values[p] <- result[[1]]  # extract first element (e.g., rms, std, or area)
    }
    precision <- call_summary_fun(aggregation_fun, values)
  } else {
    # If too few samples in data
    precision <- NA_real_
  }

  precision
}


#' @title R6 Screen Configuration Class
#'
#' @description Provides methods for converting between pixel, millimeter, and degree units.
#'
#' @examples
#' sc <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
#' sc$pix_to_deg(960, 540)
#' @export
#' @importFrom R6 R6Class
ScreenConfiguration <- R6Class("ScreenConfiguration",
  public = list(
    #' @field screen_size_x_mm Screen width in mm.
    #' @field screen_size_y_mm Screen height in mm.
    #' @field screen_res_x_pix Horizontal screen resolution in pixels.
    #' @field screen_res_y_pix Vertical screen resolution in pixels.
    #' @field viewing_distance_mm Viewing distance in mm.
    screen_size_x_mm = NULL,
    screen_size_y_mm = NULL,
    screen_res_x_pix = NULL,
    screen_res_y_pix = NULL,
    viewing_distance_mm = NULL,

    #' @description Creates a new ScreenConfiguration object with screen and viewing distance parameters.
    #' @param screen_size_x_mm Screen width in millimeters.
    #' @param screen_size_y_mm Screen height in millimeters.
    #' @param screen_res_x_pix Horizontal screen resolution in pixels.
    #' @param screen_res_y_pix Vertical screen resolution in pixels.
    #' @param viewing_distance_mm Viewing distance in millimeters.
    #' @return A new ScreenConfiguration object.
    #' @examples
    #' sc <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
    initialize = function(screen_size_x_mm, screen_size_y_mm, screen_res_x_pix, screen_res_y_pix, viewing_distance_mm) {
      self$screen_size_x_mm <- screen_size_x_mm
      self$screen_size_y_mm <- screen_size_y_mm
      self$screen_res_x_pix <- screen_res_x_pix
      self$screen_res_y_pix <- screen_res_y_pix
      self$viewing_distance_mm <- viewing_distance_mm
    },

    #' @description Converts pixel coordinates to millimeter coordinates on the screen.
    #' @param x Horizontal pixel coordinate.
    #' @param y Vertical pixel coordinate.
    #' @return A list with x and y in millimeters.
    #' @examples
    #' sc$pix_to_mm(960, 540)
    pix_to_mm = function(x, y) {
      x_mm <- x / self$screen_res_x_pix * self$screen_size_x_mm
      y_mm <- y / self$screen_res_y_pix * self$screen_size_y_mm
      list(x = x_mm, y = y_mm)
    },

    #' @description Converts pixel coordinates to an angular gaze direction in degrees.
    #' @param x Horizontal pixel coordinate.
    #' @param y Vertical pixel coordinate.
    #' @return A list with azimuth (\code{"azi"}) and elevation (\code{"ele"}) in degrees.
    #' @examples
    #' sc$pix_to_deg(960, 540)
    pix_to_deg = function(x, y) {
      mm <- self$pix_to_mm(x, y)
      self$mm_to_deg(mm$x, mm$y)
    },

    #' @description Converts millimeter coordinates to an angular gaze direction in degrees.
    #' @param x Horizontal position in millimeters.
    #' @param y Vertical position in millimeters.
    #' @return A list with azimuth (\code{"azi"}) and elevation (\code{"ele"}) in degrees.
    #' @examples
    #' sc$mm_to_deg(100, 50)
    mm_to_deg = function(x, y) {
      azi <- atan2(x, self$viewing_distance_mm) * 180 / pi
      ele <- atan2(y, sqrt(self$viewing_distance_mm^2 + x^2)) * 180 / pi
      list(azi = azi, ele = ele)
    },

    #' @description Converts millimeter coordinates on the screen to pixel coordinates.
    #' @param x Horizontal position in millimeters.
    #' @param y Vertical position in millimeters.
    #' @return A list with x and y in pixels.
    #' @examples
    #' sc$mm_to_pix(100, 50)
    mm_to_pix = function(x, y) {
      x_pix <- x / self$screen_size_x_mm * self$screen_res_x_pix
      y_pix <- y / self$screen_size_y_mm * self$screen_res_y_pix
      list(x = x_pix, y = y_pix)
    },

    #' @description Converts an angular gaze direction in degrees to millimeter coordinates on the screen.
    #' @param azi Azimuth in degrees (Fick angles).
    #' @param ele Elevation in degrees (Fick angles).
    #' @return A list with x and y in millimeters.
    #' @examples
    #' sc$deg_to_mm(2, 1)
    deg_to_mm = function(azi, ele) {
      x_mm <- self$viewing_distance_mm * tan(azi * pi / 180)
      y_mm <- self$viewing_distance_mm * tan(ele * pi / 180) / cos(azi * pi / 180)
      list(x = x_mm, y = y_mm)
    },

    #' @description Converts an angular gaze direction in degrees to pixel coordinates.
    #' @param azi Azimuth in degrees (Fick angles).
    #' @param ele Elevation in degrees (Fick angles).
    #' @return A list with x and y in pixels.
    #' @examples
    #' sc$deg_to_pix(2, 1)
    deg_to_pix = function(azi, ele) {
      mm <- self$deg_to_mm(azi, ele)
      self$mm_to_pix(mm$x, mm$y)
    },

    #' @description Computes the horizontal and vertical extents of the screen (in degrees).
    #' @return A list with width and height in degrees.
    #' @examples
    #' sc$screen_extents()
    screen_extents = function() {
      w <- self$mm_to_deg(self$screen_size_x_mm / 2, 0.)
      h <- self$mm_to_deg(0., self$screen_size_y_mm / 2)
      list(width = w$azi * 2, height = h$ele * 2)
    }
  )
)


#' @title R6 class for calculating Data Quality from a gaze data segment
#'
#' @description Provides methods for assessing the quality of gaze data, including accuracy, precision, data loss, and effective sampling frequency.
#'
#' @examples
#' sc <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
#' gaze_x <- c(0, 1, -1)
#' gaze_y <- c(0, 1, -1)
#' timestamps <- c(0, 1, 2)
#' dq <- DataQuality$new(gaze_x, gaze_y, timestamps, unit = "pixels", screen = sc)
#' dq$accuracy(0, 0)
#' dq$precision_RMS_S2S()
#' dq$data_loss_from_invalid()
#'
#' @export
#' @importFrom R6 R6Class
DataQuality <- R6Class("DataQuality",
  public = list(
    #' @field timestamps Vector of timestamps in seconds. Samples with missing data should not be removed, or the RMS calculation would be incorrect.
    #' @field azi Vector of azimuth angles in degrees (Fick angles). Missing data should be coded as NA, not using some special value such as (0,0) or (-xres,-yres).
    #' @field ele Vector of elevation angles in degrees (Fick angles). Missing data should be coded as NA, not using some special value such as (0,0) or (-xres,-yres).
    timestamps = NULL,
    azi = NULL,
    ele = NULL,

    #' @description Creates a new DataQuality object from gaze data and timestamps.
    #' @param gaze_x Horizontal gaze positions (pixels or degrees).
    #' @param gaze_y Vertical gaze positions (pixels or degrees).
    #' @param timestamps Vector of timestamps in seconds.
    #' @param unit Unit of gaze data: either \code{"pixels"} or \code{"degrees"}.
    #' @param screen Optional \code{ScreenConfiguration} object, required if unit is \code{"pixels"}.
    #' @return A new DataQuality object.
    #' @examples
    #' dq <- DataQuality$new(gaze_x, gaze_y, timestamps, unit = "pixels", screen = sc)
    initialize = function(gaze_x, gaze_y, timestamps, unit, screen = NULL) {
      self$timestamps <- as.numeric(timestamps)

      gaze_x <- as.numeric(gaze_x)
      gaze_y <- as.numeric(gaze_y)

      if (unit == "pixels") {
        if (is.null(screen)) {
          stop('If unit is "pixels", a screen configuration must be supplied')
        }
        deg <- screen$pix_to_deg(gaze_x, gaze_y)
        gaze_x <- deg$azi
        gaze_y <- deg$ele
      } else if (unit != "degrees") {
        stop('unit should be "pixels" or "degrees"')
      }

      self$azi <- gaze_x
      self$ele <- gaze_y
    },

    #' @description Calculates the accuracy of gaze data relative to a known target location.
    #' @param target_azi Target azimuth in degrees.
    #' @param target_ele Target elevation in degrees.
    #' @param central_tendency_fun Function to compute central tendency (e.g., \code{mean}, \code{median}).
    #' @return Accuracy in degrees.
    #' @examples
    #' dq$accuracy(0, 0)
    accuracy = function(target_azi, target_ele, central_tendency_fun = mean) {
      accuracy(self$azi, self$ele, target_azi, target_ele, central_tendency_fun)
    },

    #' @description Calculates precision as root mean square of sample-to-sample distances
    #' @param central_tendency_fun Function to compute central tendency (e.g., \code{mean}, \code{median}).
    #' @return Precision in degrees.
    #' @examples
    #' dq$precision_RMS_S2S()
    precision_RMS_S2S = function(central_tendency_fun = mean) {
      rms_s2s(self$azi, self$ele, central_tendency_fun)
    },

    #' @description Calculates precision as standard deviation of gaze positions.
    #' @return Standard deviation in degrees.
    #' @examples
    #' dq$precision_STD()
    precision_STD = function() {
      std(self$azi, self$ele)
    },

    #' @description Calculates the Bivariate Contour Ellipse Area (BCEA) and ellipse parameters for gaze precision.
    #' @param P Proportion of data to include in the ellipse (default is 0.68).
    #' @return BCEA in degrees-squared.
    #' @examples
    #' dq$precision_BCEA()
    precision_BCEA = function(P = 0.68) {
      bcea(self$azi, self$ele, P)
    },

    #' @description Calculates the proportion of missing data (coded as NA).
    #' @return Proportion of missing samples.
    #' @examples
    #' dq$data_loss_from_invalid()
    data_loss_from_invalid = function() {
      data_loss_from_invalid(self$azi, self$ele)
    },

    #' @description Estimates data loss based on expected number of samples given the duration and sampling frequency.
    #' @param frequency Expected sampling frequency in Hz.
    #' @return Proportion of missing samples.
    #' @examples
    #' dq$data_loss_from_expected(500)
    data_loss_from_expected = function(frequency) {
      data_loss_from_expected(self$azi, self$ele, self$get_duration(), frequency)
    },

    #' @description Calculates the effective sampling frequency based on timestamps.
    #' @return Effective frequency in Hz.
    #' @examples
    #' dq$effective_frequency()
    effective_frequency = function() {
      effective_frequency(self$azi, self$ele, self$get_duration())
    },

    #' @description Computes the total duration of the gaze recording, including the last sample.
    #' @return Duration in seconds.
    #' @examples
    #' dq$get_duration()
    get_duration = function() {
      # to get duration right, we need to include duration of last sample
      isi <- median(diff(self$timestamps), na.rm = TRUE)
      self$timestamps[length(self$timestamps)] - self$timestamps[1] + isi
    },

    #' @description Calculates precision using a moving window approach.
    #' @param window_length Length of the moving window in number of samples.
    #' @param metric Precision metric to use (\code{"RMS-S2S"}, \code{"STD"}, or \code{"BCEA"}).
    #' @param aggregation_fun Function to aggregate windowed precision values (e.g., \code{median}).
    #' @param ... Additional arguments passed to the precision metric function.
    #' @return Precision value.
    #' @examples
    #' dq$precision_using_moving_window(20, "RMS-S2S")
    precision_using_moving_window = function(window_length, metric, aggregation_fun = median, ...) {
      precision_using_moving_window(self$azi, self$ele, window_length, metric, aggregation_fun, ...)
    }
  )
)


#' Compute Data Quality Metrics from Validation Data
#'
#' This function computes a set of data quality metrics for gaze data collected during the PsychoPy validation procedure
#' that is provided in the ETDQualitizer repository on github
#' (https://github.com/dcnieho/ETDQualitizer/tree/master/python/ETDQualitizer/stim).
#' It evaluates accuracy, precision, and optionally data loss and effective sampling frequency, per eye and per target.
#'
#' @param gaze A `data.frame` containing gaze data. Must include columns `target_id`, `tar_x`, `tar_y`, `timestamp`,
#'   and eye-specific columns such as `left_x`, `left_y`, `right_x`, `right_y`. Timestamps should be provided in milliseconds.
#' @param unit A character string specifying the unit of measurement for gaze and target coordinates in the gaze data.frame.
#'   Must be either `"pixels"` or `"degrees"`.
#' @param screen An optional `ScreenConfiguration` object or numeric scalar used to convert pixel coordinates to degrees.
#'   Required if `unit == "pixels"`.
#' @param advanced Logical. If `TRUE`, all available metrics are returned. If `FALSE`, only a simplified subset is included (default is FALSE).
#' @param include_data_loss Logical. If `TRUE`, includes data loss and effective frequency metrics in the output (default is FALSE).
#'
#' @return A `data.frame` with one row per eye-target combination, containing computed metrics:
#'   - `eye`, `target_id`: identifiers
#'   - `accuracy`, `accuracy_x`, `accuracy_y`: accuracy metrics (`accuracy_x`, `accuracy_y` only if `advanced` is `TRUE`)
#'   - `rms_s2s`, `rms_s2s_x`, `rms_s2s_y`: precision (RMS sample-to-sample) (`rms_s2s_x`, `rms_s2s_y` only if `advanced` is `TRUE`)
#'   - `std`, `std_x`, `std_y`: precision (standard deviation) (`std_x`, `std_y` only if `advanced` is `TRUE`)
#'   - `bcea`, `bcea_orientation`, `bcea_ax1`, `bcea_ax2`, `bcea_aspect_ratio`: precision (BCEA metrics) (`bcea_orientation`, `bcea_ax1`, `bcea_ax2`, `bcea_aspect_ratio` only if `advanced` is `TRUE`)
#'   - `data_loss`, `effective_frequency`: optional metrics if `include_data_loss = TRUE`
#'
#' @details
#' This function uses the following methods in the `DataQuality` class to compute the returned results:
#' `accuracy()`, `precision_RMS_S2S()`, `precision_STD()`, `precision_BCEA()`, `data_loss_from_invalid()`, and `effective_frequency()`.
#'
#' @examples
#' \dontrun{
#' # NB: this example requires a gaze data table to run. See the complete example at
#' # https://github.com/dcnieho/ETDQualitizer/blob/master/example/R.R for how to prepare
#' # the input data for this function
#' dq <- compute_data_quality_from_validation(gaze_data, unit = "pixels", screen = my_screen_config)
#' }
#'
#' @export
compute_data_quality_from_validation <- function(gaze, unit, screen = NULL, advanced = FALSE, include_data_loss = FALSE) {
  stopifnot(is.data.frame(gaze))
  stopifnot(unit %in% c("pixels", "degrees"))

  # Get all targets
  targets <- unique(gaze$target_id)
  targets <- targets[targets != -1]
  target_locations <- matrix(NA, nrow = length(targets), ncol = 2)

  for (t in seq_along(targets)) {
    qTarget <- gaze$target_id == targets[t]
    iTarget <- which(qTarget)[1]
    target_locations[t, ] <- c(gaze$tar_x[iTarget], gaze$tar_y[iTarget])
  }

  # Convert target locations to degrees if needed
  if (unit == "pixels") {
    if (is.null(screen)) {
      stop('If unit is "pixels", a screen configuration must be supplied')
    }
    deg_coords <- screen$pix_to_deg(target_locations[, 1], target_locations[, 2])
    target_locations[, 1] <- deg_coords$azi
    target_locations[, 2] <- deg_coords$ele
  } else if (unit != "degrees") {
    stop('unit should be "pixels" or "degrees"')
  }

  # Determine which eyes are present
  eyes <- c("left", "right")
  have_eye <- sapply(eyes, function(e) paste0(e, "_x") %in% names(gaze))
  eyes <- eyes[have_eye]

  # Prepare output table
  vars <- c("eye", "target_id", "accuracy", "accuracy_x", "accuracy_y",
            "rms_s2s", "rms_s2s_x", "rms_s2s_y",
            "std", "std_x", "std_y",
            "bcea", "bcea_orientation", "bcea_ax1", "bcea_ax2", "bcea_aspect_ratio")
  if (include_data_loss) {
    vars <- c(vars, "data_loss", "effective_frequency")
  }

  dq <- data.frame(matrix(NA, nrow = length(targets) * length(eyes), ncol = length(vars)))
  names(dq) <- vars
  dq$eye <- ""
  dq$target_id <- NA

  # Compute metrics per eye and target
  for (e in seq_along(eyes)) {
    eye <- eyes[e]
    x_col <- paste0(eye, "_x")
    y_col <- paste0(eye, "_y")

    for (t in seq_along(targets)) {
      oi <- (e - 1) * length(targets) + t
      t_id <- targets[t]
      is_target <- gaze$target_id == t_id

      dq_calc <- DataQuality$new(
        gaze_x = gaze[[x_col]][is_target],
        gaze_y = gaze[[y_col]][is_target],
        timestamps = gaze$timestamp[is_target] / 1000,
        unit = unit,
        screen = screen
      )

      dq$eye[oi] <- eye
      dq$target_id[oi] <- t_id
      dq[oi, c("accuracy", "accuracy_x", "accuracy_y")] <- dq_calc$accuracy(target_locations[t, 1], target_locations[t, 2])
      dq[oi, c("rms_s2s", "rms_s2s_x", "rms_s2s_y")] <- dq_calc$precision_RMS_S2S()
      dq[oi, c("std", "std_x", "std_y")] <- dq_calc$precision_STD()
      dq[oi, c("bcea", "bcea_orientation", "bcea_ax1", "bcea_ax2", "bcea_aspect_ratio")] <- dq_calc$precision_BCEA()

      if (include_data_loss) {
        dq$data_loss[oi] <- dq_calc$data_loss_from_invalid()
        dq$effective_frequency[oi] <- dq_calc$effective_frequency()
      }
    }
  }

  # Drop advanced metrics if not requested
  if (!advanced) {
    keep_vars <- c("eye", "target_id", "accuracy", "rms_s2s", "std", "bcea")
    if (include_data_loss) {
      keep_vars <- c(keep_vars, "data_loss", "effective_frequency")
    }
    dq <- dq[, keep_vars, drop = FALSE]
  }

  dq
}


#' Summarize and Report Data Quality Metrics
#'
#' This function summarizes data quality metrics from a validation procedure by computing averages per participant and generating descriptive statistics across participants.
#' It also returns a formatted textual summary suitable for reporting.
#'
#' @param dq_table A `data.frame` containing data quality metrics. Must include columns `file`, `eye`, `target_id`, and relevant numeric metrics such as `accuracy`, `rms_s2s`, and `std`.
#'  This would generally be created by concatenating the output of the compute_data_quality_from_validation() for multiple files.
#'
#' @return A named list with two elements:
#' \describe{
#'   \item{txt}{A character string summarizing key metrics (accuracy, RMS-S2S precision, STD precision).}
#'   \item{measures}{A list containing:
#'     \itemize{
#'       \item{\code{all}: A data frame with per-participant averages (grouped by `file`).}
#'       \item{\code{mean}, \code{std}, \code{min}, \code{max}: Named numeric vectors with summary statistics across participants.}
#'     }
#'   }
#' }
#'
#' @details
#' The summary text excludes BCEA and data loss metrics. BCEA is considered a niche metric and data loss is best reported across the full dataset rather than just the validation subset.
#'
#' @examples
#' \dontrun{
#' # NB: this example requires a gaze data table to run. See the complete example at
#' # https://github.com/dcnieho/ETDQualitizer/blob/master/example/R.R for how to prepare
#' # the input data for this function
#' result <- report_data_quality_table(dq_table)
#' cat(result$txt)
#' head(result$measures$all)
#' }
#'
#' @export
#' @importFrom stats aggregate
report_data_quality_table <- function(dq_table) {
  stopifnot(is.data.frame(dq_table))

  measures <- list()

  # Average over targets and eyes, grouped by file
  grouped <- aggregate(. ~ file, data = dq_table[, !(names(dq_table) %in% c("eye", "target_id"))], FUN = mean, na.rm = TRUE)
  measures$all <- grouped

  # Summary statistics
  numeric_cols <- names(grouped)[-1]  # exclude 'file'
  measures$mean <- sapply(grouped[numeric_cols], mean, na.rm = TRUE)
  measures$std  <- sapply(grouped[numeric_cols], sd, na.rm = TRUE)
  measures$min  <- sapply(grouped[numeric_cols], min, na.rm = TRUE)
  measures$max  <- sapply(grouped[numeric_cols], max, na.rm = TRUE)

  # Text summary (excluding BCEA and data loss)
  n_target <- length(unique(dq_table$target_id))
  n_subj   <- nrow(measures$all)
  version  <- ETDQ_version()  # Assumes this function exists

  txt <- sprintf(
    "For %d participants, the average inaccuracy in the data determined from a %d-point validation procedure using ETDQualitizer v%s (Niehorster et al., 2026) was %.2f\u00b0 (SD=%.2f\u00b0, range=%.2f\u00b0--%.2f\u00b0). Average RMS-S2S precision was %.3f\u00b0 (SD=%.3f\u00b0, range=%.3f\u00b0--%.3f\u00b0) and STD precision %.3f\u00b0 (SD=%.3f\u00b0, range=%.3f\u00b0--%.3f\u00b0).",
    n_subj, n_target, version,
    measures$mean["accuracy"], measures$std["accuracy"], measures$min["accuracy"], measures$max["accuracy"],
    measures$mean["rms_s2s"], measures$std["rms_s2s"], measures$min["rms_s2s"], measures$max["rms_s2s"],
    measures$mean["std"], measures$std["std"], measures$min["std"], measures$max["std"]
  )

  list(txt = txt, measures = measures)
}

test_that("Fick_to_vector with default radius", {
  vec <- Fick_to_vector(0, 0)
  expect_equal(vec$x, 0, tolerance = 1e-8)
  expect_equal(vec$y, 0, tolerance = 1e-8)
  expect_equal(vec$z, 1, tolerance = 1e-8)
})

test_that("Fick_to_vector with custom radius", {
  vec <- Fick_to_vector(90, 0, 2)
  expect_equal(vec$x, 2, tolerance = 1e-8)
  expect_equal(vec$y, 0, tolerance = 1e-8)
  expect_equal(vec$z, 0, tolerance = 1e-8)
})

test_that("vector_to_Fick basic conversion", {
  fick <- vector_to_Fick(0, 0, 1)
  expect_equal(fick$azi, 0, tolerance = 1e-8)
  expect_equal(fick$ele, 0, tolerance = 1e-8)
})

test_that("Fick round-trip conversion", {
  azi <- 45
  ele <- 30
  r <- 1.5
  vec <- Fick_to_vector(azi, ele, r)
  fick2 <- vector_to_Fick(vec$x, vec$y, vec$z)
  expect_equal(fick2$azi, azi, tolerance = 1e-5)
  expect_equal(fick2$ele, ele, tolerance = 1e-5)
})

test_that("vector_to_Fick with negative components", {
  fick <- vector_to_Fick(-1, -1, -1)
  expected_azi <- atan2(-1, -1) * 180 / pi
  expected_ele <- atan2(-1, sqrt((-1)^2 + (-1)^2)) * 180 / pi
  expect_equal(fick$azi, expected_azi, tolerance = 1e-8)
  expect_equal(fick$ele, expected_ele, tolerance = 1e-8)
})

test_that("accuracy returns zero offset for symmetric gaze around target", {
  x <- c(0, 1, -1)
  y <- c(0, 1, -1)
  result <- accuracy(x, y, 0, 0)
  expect_equal(result$offset, 0, tolerance = 1e-8)
  expect_equal(result$offset_azi, 0, tolerance = 1e-8)
  expect_equal(result$offset_ele, 0, tolerance = 1e-8)
})

test_that("accuracy works with custom central tendency function", {
  x <- c(0, 1, -1)
  y <- c(0, 1, -1)
  result <- accuracy(x, y, 0, 0, central_tendency_fun = median)
  expect_equal(result$offset, 0, tolerance = 1e-8)
  expect_equal(result$offset_azi, 0, tolerance = 1e-8)
  expect_equal(result$offset_ele, 0, tolerance = 1e-8)
})

test_that("std returns correct values", {
  x <- c(1, 2, 3)
  y <- c(4, 5, 6)
  result <- std(x, y)
  expect_equal(result$std_azi, sqrt(mean((x - mean(x))^2)))
  expect_equal(result$std_ele, sqrt(mean((y - mean(y))^2)))
  expect_equal(result$std, sqrt(result$std_a^2 + result$std_e^2))
})

test_that("std returns correct values (with NA)", {
  x <- c(1, 2, NA, 3)
  y <- c(4, 5, NA, 6)
  result <- std(x, y)
  expect_equal(result$std_azi, sqrt(mean((x - mean(x, na.rm = TRUE))^2, na.rm = TRUE)))
  expect_equal(result$std_ele, sqrt(mean((y - mean(y, na.rm = TRUE))^2, na.rm = TRUE)))
  expect_equal(result$std, sqrt(result$std_a^2 + result$std_e^2))
})

test_that("bcea returns valid area and aspect ratio", {
  set.seed(42)
  x <- rnorm(100000)
  y <- rnorm(100000)
  result <- bcea(x, y)
  expect_gt(result$area, 0)
  expect_equal(result$aspect_ratio, 1, tolerance = 0.1)
  expect_equal(result$area, 2 * pi * result$ax1 * result$ax2, tolerance = 1e-3)
})

test_that("rms_s2s returns correct RMS values", {
  x <- c(1, 2, 3)
  y <- c(4, 5, 6)
  result <- rms_s2s(x, y)
  expect_gte(result$rms, 0)
  expect_equal(result$rms_azi, sqrt(mean(diff(x)^2)))
  expect_equal(result$rms_ele, sqrt(mean(diff(y)^2)))
})

test_that("data_loss_from_invalid computes correct percentage", {
  x <- c(1, NA, 3)
  y <- c(4, 5, NA)
  loss <- data_loss_from_invalid(x, y)
  expect_equal(loss, 2 / 3 * 100)
})

test_that("data_loss_from_expected computes correct loss", {
  x <- c(1, NA, 3)
  y <- c(4, 5, NA)
  loss <- data_loss_from_expected(x, y, duration = 1, frequency = 3)
  expect_equal(loss, (1 - 1 / 3) * 100)
})

test_that("effective_frequency computes correct value", {
  x <- c(1, NA, 3)
  y <- c(4, 5, NA)
  freq <- effective_frequency(x, y, duration = 1)
  expect_equal(freq, 1)
})

test_that("constructor sets fields correctly", {
  sc <- ScreenConfiguration$new(400, 250, 1600, 900, 500)
  expect_equal(sc$screen_size_x_mm, 400)
  expect_equal(sc$screen_size_y_mm, 250)
  expect_equal(sc$screen_res_x_pix, 1600)
  expect_equal(sc$screen_res_y_pix, 900)
  expect_equal(sc$viewing_distance_mm, 500)
})

test_that("pix_to_mm conversion is correct", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  mm <- config$pix_to_mm(960, 540)
  expect_equal(mm$x, 250, tolerance = 1e-8)
  expect_equal(mm$y, 150, tolerance = 1e-8)
})

test_that("mm_to_pix conversion is correct", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  pix <- config$mm_to_pix(250, 150)
  expect_equal(pix$x, 960, tolerance = 1e-8)
  expect_equal(pix$y, 540, tolerance = 1e-8)
})

test_that("mm_to_deg conversion is correct", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  deg <- config$mm_to_deg(250, 0)
  expect_equal(deg$azi, atan2(250, 600) * 180 / pi, tolerance = 1e-8)
  expect_equal(deg$ele, 0, tolerance = 1e-8)
})

test_that("deg_to_mm conversion is correct", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  azi <- atan2(250, 600) * 180 / pi
  ele <- 0
  mm <- config$deg_to_mm(azi, ele)
  expect_equal(mm$x, 250, tolerance = 1e-8)
  expect_equal(mm$y, 0, tolerance = 1e-8)
})

test_that("pix_to_deg returns correct azimuth and elevation", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  deg <- config$pix_to_deg(960, 540)
  expected_azi <- atan2(250, 600) * 180 / pi
  expected_ele <- atan2(150, sqrt(600^2 + 250^2)) * 180 / pi
  expect_equal(deg$azi, expected_azi, tolerance = 1e-10)
  expect_equal(deg$ele, expected_ele, tolerance = 1e-10)
})

test_that("deg_to_pix conversion is correct", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  azi <- atan2(250, 600) * 180 / pi
  ele <- atan2(150, sqrt(600^2 + 250^2)) * 180 / pi
  pix <- config$deg_to_pix(azi, ele)
  expect_equal(pix$x, 960, tolerance = 1e-1)
  expect_equal(pix$y, 540, tolerance = 1e-1)
})

test_that("pix_to_mm and mm_to_pix are consistent", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  mm <- config$pix_to_mm(960, 540)
  pix <- config$mm_to_pix(mm$x, mm$y)
  expect_equal(pix$x, 960, tolerance = 1e-10)
  expect_equal(pix$y, 540, tolerance = 1e-10)
})
test_that("mm_to_deg and deg_to_mm are consistent", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  deg <- config$mm_to_deg(250, 0)
  mm <- config$deg_to_mm(deg$azi, deg$ele)
  expect_equal(mm$x, 250, tolerance = 1e-10)
  expect_equal(mm$y,   0, tolerance = 1e-10)
})

test_that("screen_extents returns correct values", {
  config <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
  extents <- config$screen_extents()
  expected_x_deg <- 2 * atan2(250, 600) * 180 / pi
  expected_y_deg <- 2 * atan2(150, 600) * 180 / pi
  expect_equal(extents$width, expected_x_deg, tolerance = 1e-8)
  expect_equal(extents$height, expected_y_deg, tolerance = 1e-8)
})

test_that("constructor with degrees stores data correctly", {
  duration <- 1000
  freq <- 100
  timestamps <- seq(0, duration - 1 / freq, by = 1 / freq)
  n_samples <- length(timestamps)
  azi <- rnorm(n_samples)
  ele <- rnorm(n_samples)

  dq <- DataQuality$new(azi, ele, timestamps, "degrees")
  expect_equal(dq$azi, azi)
  expect_equal(dq$ele, ele)
  expect_equal(dq$timestamps, timestamps)
})

test_that("constructor with pixels converts correctly", {
  duration <- 1000
  freq <- 100
  timestamps <- seq(0, duration - 1 / freq, by = 1 / freq)
  n_samples <- length(timestamps)
  azi <- rnorm(n_samples)
  ele <- rnorm(n_samples)
  screen <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)

  x_pix <- 960 + azi * 10
  y_pix <- 540 + ele * 10
  dq <- DataQuality$new(x_pix, y_pix, timestamps, "pixels", screen)
  deg <- screen$pix_to_deg(x_pix, y_pix)
  expect_equal(dq$azi, deg$azi, tolerance = 1e-8)
  expect_equal(dq$ele, deg$ele, tolerance = 1e-8)
})

test_that("accuracy returns near-zero offset for symmetric data", {
  dq <- DataQuality$new(c(0, 1, -1), c(0, 1, -1), c(0, 1, 2), "degrees")
  result <- dq$accuracy(0, 0)
  expect_equal(result$offset, 0, tolerance = 0.1)
  expect_equal(result$offset_azi, 0, tolerance = 0.1)
  expect_equal(result$offset_ele, 0, tolerance = 0.1)
})

test_that("precision RMS with large dataset", {
  dq <- DataQuality$new(rnorm(100000), rnorm(100000), seq(0, 1000, length.out = 100000), "degrees")
  result <- dq$precision_RMS_S2S()
  expect_equal(result$rms, 2, tolerance = 0.1)
  expect_equal(result$rms_azi, sqrt(2), tolerance = 0.1)
  expect_equal(result$rms_ele, sqrt(2), tolerance = 0.1)
})

test_that("precision RMS with one axis varying", {
  dq <- DataQuality$new(c(1, 2, 4), c(1, 1, 1), c(0, 1, 2), "degrees")
  result <- dq$precision_RMS_S2S()
  expected_rms <- sqrt(mean(c(1, 2)^2))
  expect_gte(result$rms, expected_rms)
  expect_gte(result$rms_azi, expected_rms)
  expect_gte(result$rms_ele, 0)
})

test_that("precision RMS with both axes varying", {
  dq <- DataQuality$new(c(1, 2, 4), c(1, 2, 4), c(0, 1, 2), "degrees")
  result <- dq$precision_RMS_S2S()
  expected_rms_xy <- sqrt(mean(c(1, 2)^2))
  expected_rms <- sqrt(2) * expected_rms_xy
  expect_gte(result$rms, expected_rms)
  expect_gte(result$rms_azi, expected_rms_xy)
  expect_gte(result$rms_ele, expected_rms_xy)
})

test_that("precision STD returns expected values", {
  dq <- DataQuality$new(rnorm(100000, sd = 1), rnorm(100000, sd = 1), seq(0, 1000, length.out = 100000), "degrees")
  result <- dq$precision_STD()
  expect_equal(result$std, sqrt(2), tolerance = 0.1)
  expect_equal(result$std_azi, 1, tolerance = 0.1)
  expect_equal(result$std_ele, 1, tolerance = 0.1)
})

test_that("precision BCEA returns valid values", {
  dq <- DataQuality$new(rnorm(10000), rnorm(10000), seq(0, 100, length.out = 10000), "degrees")
  result <- dq$precision_BCEA()
  expect_gt(result$area, 0)
  expect_equal(result$aspect_ratio, 1, tolerance = 0.1)
  expect_equal(result$area, 2 * pi * result$ax1 * result$ax2, tolerance = 1e-3)
})

test_that("precision STD using moving window returns reasonable value", {
  dq <- DataQuality$new(rnorm(10000), rnorm(10000), seq(0, 100, length.out = 10000), "degrees")
  s <- dq$precision_using_moving_window(50, "STD")
  expect_lte(s, sqrt(2))
})

test_that("data loss is zero for complete data", {
  dq <- DataQuality$new(rnorm(1000), rnorm(1000), seq(0, 10, length.out = 1000), "degrees")
  expect_equal(dq$data_loss_from_invalid(), 0)
})

test_that("data loss from expected is zero for complete data", {
  freq <- 100
  timestamps <- seq(0, 10, by = 1 / freq)
  dq <- DataQuality$new(rnorm(length(timestamps)), rnorm(length(timestamps)), timestamps, "degrees")
  expect_equal(dq$data_loss_from_expected(freq), 0)
})

test_that("effective frequency matches expected", {
  freq <- 100
  timestamps <- seq(0, 10 - 1 / freq, by = 1 / freq)
  dq <- DataQuality$new(rnorm(length(timestamps)), rnorm(length(timestamps)), timestamps, "degrees")
  expect_equal(dq$effective_frequency(), freq)
})

test_that("get_duration returns correct value", {
  duration <- 1000
  freq <- 100
  timestamps <- seq(0, duration - 1 / freq, by = 1 / freq)
  dq <- DataQuality$new(rnorm(length(timestamps)), rnorm(length(timestamps)), timestamps, "degrees")
  expect_equal(dq$get_duration(), duration, tolerance = 1e-6)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{Fick_to_vector}
\alias{Fick_to_vector}
\title{Convert Fick Angles to 3D Vector}
\usage{
Fick_to_vector(azi, ele, rho = 1)
}
\arguments{
\item{azi}{Azimuth angle in degrees.}

\item{ele}{Elevation angle in degrees.}

\item{rho}{Radius (default is 1.0).}
}
\value{
A list with components \code{x}, \code{y}, and \code{z}.
}
\description{
Converts azimuth and elevation angles (in degrees) to a 3D unit vector.
}
\examples{
Fick_to_vector(30, 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{vector_to_Fick}
\alias{vector_to_Fick}
\title{Convert 3D Vector to Fick Angles}
\usage{
vector_to_Fick(x, y, z)
}
\arguments{
\item{x}{X component of the vector.}

\item{y}{Y component of the vector.}

\item{z}{Z component of the vector.}
}
\value{
A list with components \code{azi} and \code{ele}.
}
\description{
Converts a 3D vector to azimuth and elevation angles (in degrees).
}
\examples{
vector_to_Fick(0.5, 0.2, 0.8)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{rms_s2s}
\alias{rms_s2s}
\title{RMS of Sample-to-Sample Differences}
\usage{
rms_s2s(azi, ele, central_tendency_fun = mean)
}
\arguments{
\item{azi}{Azimuth values in degrees.}

\item{ele}{Elevation values in degrees.}

\item{central_tendency_fun}{Function to compute central tendency (default: \code{mean}).}
}
\value{
A list with \code{rms}, \code{rms_azi}, and \code{rms_ele}, the total RMS of sample-to-sample distances and that of the azimuthal and elevation components (all in degrees).
}
\description{
Computes root mean square of differences between successive gaze samples.
}
\examples{
rms_s2s(c(1, 2, 3), c(1, 2, 3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{ETDQ_version}
\alias{ETDQ_version}
\title{Get ETDQualitizer Version}
\usage{
ETDQ_version()
}
\value{
A character string representing the version number.
}
\description{
Returns the current version string of the ETDQualitizer tool.
}
\examples{
ETDQ_version()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{data_loss_from_expected}
\alias{data_loss_from_expected}
\title{Compute Data Loss from Expected Sample Count}
\usage{
data_loss_from_expected(a, b, duration, frequency)
}
\arguments{
\item{a}{Horizontal gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).}

\item{b}{Vertical gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).}

\item{duration}{Duration in seconds.}

\item{frequency}{Sampling frequency in Hz.}
}
\value{
Percentage of data loss.
}
\description{
Calculates data loss based on expected number of samples.
}
\examples{
data_loss_from_expected(c(1, NA, 3), c(1, 2, NA), duration = 1, frequency = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{compute_data_quality_from_validation}
\alias{compute_data_quality_from_validation}
\title{Compute Data Quality Metrics from Validation Data}
\usage{
compute_data_quality_from_validation(
  gaze,
  unit,
  screen = NULL,
  advanced = FALSE,
  include_data_loss = FALSE
)
}
\arguments{
\item{gaze}{A `data.frame` containing gaze data. Must include columns `target_id`, `tar_x`, `tar_y`, `timestamp`,
and eye-specific columns such as `left_x`, `left_y`, `right_x`, `right_y`. Timestamps should be provided in milliseconds.}

\item{unit}{A character string specifying the unit of measurement for gaze and target coordinates in the gaze data.frame.
Must be either `"pixels"` or `"degrees"`.}

\item{screen}{An optional `ScreenConfiguration` object or numeric scalar used to convert pixel coordinates to degrees.
Required if `unit == "pixels"`.}

\item{advanced}{Logical. If `TRUE`, all available metrics are returned. If `FALSE`, only a simplified subset is included (default is FALSE).}

\item{include_data_loss}{Logical. If `TRUE`, includes data loss and effective frequency metrics in the output (default is FALSE).}
}
\value{
A `data.frame` with one row per eye-target combination, containing computed metrics:
  - `eye`, `target_id`: identifiers
  - `accuracy`, `accuracy_x`, `accuracy_y`: accuracy metrics (`accuracy_x`, `accuracy_y` only if `advanced` is `TRUE`)
  - `rms_s2s`, `rms_s2s_x`, `rms_s2s_y`: precision (RMS sample-to-sample) (`rms_s2s_x`, `rms_s2s_y` only if `advanced` is `TRUE`)
  - `std`, `std_x`, `std_y`: precision (standard deviation) (`std_x`, `std_y` only if `advanced` is `TRUE`)
  - `bcea`, `bcea_orientation`, `bcea_ax1`, `bcea_ax2`, `bcea_aspect_ratio`: precision (BCEA metrics) (`bcea_orientation`, `bcea_ax1`, `bcea_ax2`, `bcea_aspect_ratio` only if `advanced` is `TRUE`)
  - `data_loss`, `effective_frequency`: optional metrics if `include_data_loss = TRUE`
}
\description{
This function computes a set of data quality metrics for gaze data collected during the PsychoPy validation procedure
that is provided in the ETDQualitizer repository on github
(https://github.com/dcnieho/ETDQualitizer/tree/master/python/ETDQualitizer/stim).
It evaluates accuracy, precision, and optionally data loss and effective sampling frequency, per eye and per target.
}
\details{
This function uses the following methods in the `DataQuality` class to compute the returned results:
`accuracy()`, `precision_RMS_S2S()`, `precision_STD()`, `precision_BCEA()`, `data_loss_from_invalid()`, and `effective_frequency()`.
}
\examples{
\dontrun{
# NB: this example requires a gaze data table to run. See the complete example at
# https://github.com/dcnieho/ETDQualitizer/blob/master/example/R.R for how to prepare
# the input data for this function
dq <- compute_data_quality_from_validation(gaze_data, unit = "pixels", screen = my_screen_config)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{std}
\alias{std}
\title{Standard Deviation of Gaze Samples}
\usage{
std(azi, ele)
}
\arguments{
\item{azi}{Azimuth values in degrees.}

\item{ele}{Elevation values in degrees.}
}
\value{
A list with \code{std}, \code{std_azi}, and \code{std_ele}, the total STD of sample-to-sample distances and that of the azimuthal and elevation components (all in degrees).
}
\description{
Computes standard deviation of azimuth and elevation.
}
\examples{
std(c(1, 2, 3), c(1, 2, 3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{data_loss_from_invalid}
\alias{data_loss_from_invalid}
\title{Compute Data Loss from number of invalid samples.}
\usage{
data_loss_from_invalid(a, b)
}
\arguments{
\item{a}{Horizontal gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).}

\item{b}{Vertical gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).}
}
\value{
Percentage of missing samples.
}
\description{
Calculates percentage of missing gaze samples.
}
\examples{
data_loss_from_invalid(c(1, NA, 3), c(1, 2, NA))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{bcea}
\alias{bcea}
\title{Bivariate Contour Ellipse Area (BCEA)}
\usage{
bcea(azi, ele, P = 0.68)
}
\arguments{
\item{azi}{Azimuth values in degrees.}

\item{ele}{Elevation values in degrees.}

\item{P}{Cumulative probability (default: 0.68).}
}
\value{
A list with the BCEA (\code{area}) and additional info about the BCEA ellipse: \code{orientation}, \code{ax1}, \code{ax2}, and \code{aspect_ratio}.
}
\description{
Computes BCEA and ellipse parameters for gaze precision.
}
\examples{
bcea(rnorm(100), rnorm(100))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{accuracy}
\alias{accuracy}
\title{Compute Gaze Accuracy}
\usage{
accuracy(azi, ele, target_azi, target_ele, central_tendency_fun = mean)
}
\arguments{
\item{azi}{Gaze azimuth in degrees.}

\item{ele}{Gaze elevation in degrees.}

\item{target_azi}{Target azimuth in degrees.}

\item{target_ele}{Target elevation in degrees.}

\item{central_tendency_fun}{Function to compute central tendency (default: \code{mean}).}
}
\value{
A list with \code{offset}, \code{offset_azi}, and \code{offset_ele}, the total, horizontal and vertical offset of gaze from the target (in degrees).
}
\description{
Calculates the angular offset between gaze and target directions.
}
\examples{
accuracy(c(1, 2), c(1, 2), 0, 0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{effective_frequency}
\alias{effective_frequency}
\title{Compute Effective Sampling Frequency}
\usage{
effective_frequency(a, b, duration)
}
\arguments{
\item{a}{Horizontal gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).}

\item{b}{Vertical gaze values (e.g. azimuth or horizontal coordinate in pixels or mm).}

\item{duration}{Duration in seconds.}
}
\value{
Effective frequency in Hz.
}
\description{
Calculates effective frequency based on valid samples.
}
\examples{
effective_frequency(c(1, NA, 3), c(1, 2, NA), duration = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{report_data_quality_table}
\alias{report_data_quality_table}
\title{Summarize and Report Data Quality Metrics}
\usage{
report_data_quality_table(dq_table)
}
\arguments{
\item{dq_table}{A `data.frame` containing data quality metrics. Must include columns `file`, `eye`, `target_id`, and relevant numeric metrics such as `accuracy`, `rms_s2s`, and `std`.
This would generally be created by concatenating the output of the compute_data_quality_from_validation() for multiple files.}
}
\value{
A named list with two elements:
\describe{
  \item{txt}{A character string summarizing key metrics (accuracy, RMS-S2S precision, STD precision).}
  \item{measures}{A list containing:
    \itemize{
      \item{\code{all}: A data frame with per-participant averages (grouped by `file`).}
      \item{\code{mean}, \code{std}, \code{min}, \code{max}: Named numeric vectors with summary statistics across participants.}
    }
  }
}
}
\description{
This function summarizes data quality metrics from a validation procedure by computing averages per participant and generating descriptive statistics across participants.
It also returns a formatted textual summary suitable for reporting.
}
\details{
The summary text excludes BCEA and data loss metrics. BCEA is considered a niche metric and data loss is best reported across the full dataset rather than just the validation subset.
}
\examples{
\dontrun{
# NB: this example requires a gaze data table to run. See the complete example at
# https://github.com/dcnieho/ETDQualitizer/blob/master/example/R.R for how to prepare
# the input data for this function
result <- report_data_quality_table(dq_table)
cat(result$txt)
head(result$measures$all)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{precision_using_moving_window}
\alias{precision_using_moving_window}
\title{Precision Using Moving Window}
\usage{
precision_using_moving_window(
  azi,
  ele,
  window_length,
  metric,
  aggregation_fun = median,
  ...
)
}
\arguments{
\item{azi}{Azimuth values.}

\item{ele}{Elevation values.}

\item{window_length}{Window size in samples.}

\item{metric}{Precision metric: \code{"RMS-S2S"}, \code{"STD"}, or \code{"BCEA"}.}

\item{aggregation_fun}{Function to aggregate precision values across the windows (default: \code{median}).}

\item{...}{Additional arguments passed to metric function.}
}
\value{
Aggregated precision value.
}
\description{
Computes gaze precision using a moving window and selected metric.
}
\examples{
precision_using_moving_window(rnorm(100), rnorm(100), 10, "STD")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{ScreenConfiguration}
\alias{ScreenConfiguration}
\title{R6 Screen Configuration Class}
\description{
Provides methods for converting between pixel, millimeter, and degree units.
}
\examples{
sc <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
sc$pix_to_deg(960, 540)

## ------------------------------------------------
## Method `ScreenConfiguration$new`
## ------------------------------------------------

sc <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)

## ------------------------------------------------
## Method `ScreenConfiguration$pix_to_mm`
## ------------------------------------------------

sc$pix_to_mm(960, 540)

## ------------------------------------------------
## Method `ScreenConfiguration$pix_to_deg`
## ------------------------------------------------

sc$pix_to_deg(960, 540)

## ------------------------------------------------
## Method `ScreenConfiguration$mm_to_deg`
## ------------------------------------------------

sc$mm_to_deg(100, 50)

## ------------------------------------------------
## Method `ScreenConfiguration$mm_to_pix`
## ------------------------------------------------

sc$mm_to_pix(100, 50)

## ------------------------------------------------
## Method `ScreenConfiguration$deg_to_mm`
## ------------------------------------------------

sc$deg_to_mm(2, 1)

## ------------------------------------------------
## Method `ScreenConfiguration$deg_to_pix`
## ------------------------------------------------

sc$deg_to_pix(2, 1)

## ------------------------------------------------
## Method `ScreenConfiguration$screen_extents`
## ------------------------------------------------

sc$screen_extents()
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{screen_size_x_mm}}{Screen width in mm.}

\item{\code{screen_size_y_mm}}{Screen height in mm.}

\item{\code{screen_res_x_pix}}{Horizontal screen resolution in pixels.}

\item{\code{screen_res_y_pix}}{Vertical screen resolution in pixels.}

\item{\code{viewing_distance_mm}}{Viewing distance in mm.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ScreenConfiguration-new}{\code{ScreenConfiguration$new()}}
\item \href{#method-ScreenConfiguration-pix_to_mm}{\code{ScreenConfiguration$pix_to_mm()}}
\item \href{#method-ScreenConfiguration-pix_to_deg}{\code{ScreenConfiguration$pix_to_deg()}}
\item \href{#method-ScreenConfiguration-mm_to_deg}{\code{ScreenConfiguration$mm_to_deg()}}
\item \href{#method-ScreenConfiguration-mm_to_pix}{\code{ScreenConfiguration$mm_to_pix()}}
\item \href{#method-ScreenConfiguration-deg_to_mm}{\code{ScreenConfiguration$deg_to_mm()}}
\item \href{#method-ScreenConfiguration-deg_to_pix}{\code{ScreenConfiguration$deg_to_pix()}}
\item \href{#method-ScreenConfiguration-screen_extents}{\code{ScreenConfiguration$screen_extents()}}
\item \href{#method-ScreenConfiguration-clone}{\code{ScreenConfiguration$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-new"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-new}{}}}
\subsection{Method \code{new()}}{
Creates a new ScreenConfiguration object with screen and viewing distance parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$new(
  screen_size_x_mm,
  screen_size_y_mm,
  screen_res_x_pix,
  screen_res_y_pix,
  viewing_distance_mm
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{screen_size_x_mm}}{Screen width in millimeters.}

\item{\code{screen_size_y_mm}}{Screen height in millimeters.}

\item{\code{screen_res_x_pix}}{Horizontal screen resolution in pixels.}

\item{\code{screen_res_y_pix}}{Vertical screen resolution in pixels.}

\item{\code{viewing_distance_mm}}{Viewing distance in millimeters.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new ScreenConfiguration object.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{sc <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-pix_to_mm"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-pix_to_mm}{}}}
\subsection{Method \code{pix_to_mm()}}{
Converts pixel coordinates to millimeter coordinates on the screen.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$pix_to_mm(x, y)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{Horizontal pixel coordinate.}

\item{\code{y}}{Vertical pixel coordinate.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A list with x and y in millimeters.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{sc$pix_to_mm(960, 540)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-pix_to_deg"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-pix_to_deg}{}}}
\subsection{Method \code{pix_to_deg()}}{
Converts pixel coordinates to an angular gaze direction in degrees.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$pix_to_deg(x, y)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{Horizontal pixel coordinate.}

\item{\code{y}}{Vertical pixel coordinate.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A list with azimuth (\code{"azi"}) and elevation (\code{"ele"}) in degrees.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{sc$pix_to_deg(960, 540)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-mm_to_deg"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-mm_to_deg}{}}}
\subsection{Method \code{mm_to_deg()}}{
Converts millimeter coordinates to an angular gaze direction in degrees.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$mm_to_deg(x, y)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{Horizontal position in millimeters.}

\item{\code{y}}{Vertical position in millimeters.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A list with azimuth (\code{"azi"}) and elevation (\code{"ele"}) in degrees.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{sc$mm_to_deg(100, 50)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-mm_to_pix"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-mm_to_pix}{}}}
\subsection{Method \code{mm_to_pix()}}{
Converts millimeter coordinates on the screen to pixel coordinates.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$mm_to_pix(x, y)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{Horizontal position in millimeters.}

\item{\code{y}}{Vertical position in millimeters.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A list with x and y in pixels.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{sc$mm_to_pix(100, 50)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-deg_to_mm"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-deg_to_mm}{}}}
\subsection{Method \code{deg_to_mm()}}{
Converts an angular gaze direction in degrees to millimeter coordinates on the screen.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$deg_to_mm(azi, ele)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{azi}}{Azimuth in degrees (Fick angles).}

\item{\code{ele}}{Elevation in degrees (Fick angles).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A list with x and y in millimeters.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{sc$deg_to_mm(2, 1)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-deg_to_pix"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-deg_to_pix}{}}}
\subsection{Method \code{deg_to_pix()}}{
Converts an angular gaze direction in degrees to pixel coordinates.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$deg_to_pix(azi, ele)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{azi}}{Azimuth in degrees (Fick angles).}

\item{\code{ele}}{Elevation in degrees (Fick angles).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A list with x and y in pixels.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{sc$deg_to_pix(2, 1)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-screen_extents"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-screen_extents}{}}}
\subsection{Method \code{screen_extents()}}{
Computes the horizontal and vertical extents of the screen (in degrees).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$screen_extents()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A list with width and height in degrees.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{sc$screen_extents()
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ScreenConfiguration-clone"></a>}}
\if{latex}{\out{\hypertarget{method-ScreenConfiguration-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ScreenConfiguration$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ETDQualitizer.R
\name{DataQuality}
\alias{DataQuality}
\title{R6 class for calculating Data Quality from a gaze data segment}
\description{
Provides methods for assessing the quality of gaze data, including accuracy, precision, data loss, and effective sampling frequency.
}
\examples{
sc <- ScreenConfiguration$new(500, 300, 1920, 1080, 600)
gaze_x <- c(0, 1, -1)
gaze_y <- c(0, 1, -1)
timestamps <- c(0, 1, 2)
dq <- DataQuality$new(gaze_x, gaze_y, timestamps, unit = "pixels", screen = sc)
dq$accuracy(0, 0)
dq$precision_RMS_S2S()
dq$data_loss_from_invalid()


## ------------------------------------------------
## Method `DataQuality$new`
## ------------------------------------------------

dq <- DataQuality$new(gaze_x, gaze_y, timestamps, unit = "pixels", screen = sc)

## ------------------------------------------------
## Method `DataQuality$accuracy`
## ------------------------------------------------

dq$accuracy(0, 0)

## ------------------------------------------------
## Method `DataQuality$precision_RMS_S2S`
## ------------------------------------------------

dq$precision_RMS_S2S()

## ------------------------------------------------
## Method `DataQuality$precision_STD`
## ------------------------------------------------

dq$precision_STD()

## ------------------------------------------------
## Method `DataQuality$precision_BCEA`
## ------------------------------------------------

dq$precision_BCEA()

## ------------------------------------------------
## Method `DataQuality$data_loss_from_invalid`
## ------------------------------------------------

dq$data_loss_from_invalid()

## ------------------------------------------------
## Method `DataQuality$data_loss_from_expected`
## ------------------------------------------------

dq$data_loss_from_expected(500)

## ------------------------------------------------
## Method `DataQuality$effective_frequency`
## ------------------------------------------------

dq$effective_frequency()

## ------------------------------------------------
## Method `DataQuality$get_duration`
## ------------------------------------------------

dq$get_duration()

## ------------------------------------------------
## Method `DataQuality$precision_using_moving_window`
## ------------------------------------------------

dq$precision_using_moving_window(20, "RMS-S2S")
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{timestamps}}{Vector of timestamps in seconds. Samples with missing data should not be removed, or the RMS calculation would be incorrect.}

\item{\code{azi}}{Vector of azimuth angles in degrees (Fick angles). Missing data should be coded as NA, not using some special value such as (0,0) or (-xres,-yres).}

\item{\code{ele}}{Vector of elevation angles in degrees (Fick angles). Missing data should be coded as NA, not using some special value such as (0,0) or (-xres,-yres).}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-DataQuality-new}{\code{DataQuality$new()}}
\item \href{#method-DataQuality-accuracy}{\code{DataQuality$accuracy()}}
\item \href{#method-DataQuality-precision_RMS_S2S}{\code{DataQuality$precision_RMS_S2S()}}
\item \href{#method-DataQuality-precision_STD}{\code{DataQuality$precision_STD()}}
\item \href{#method-DataQuality-precision_BCEA}{\code{DataQuality$precision_BCEA()}}
\item \href{#method-DataQuality-data_loss_from_invalid}{\code{DataQuality$data_loss_from_invalid()}}
\item \href{#method-DataQuality-data_loss_from_expected}{\code{DataQuality$data_loss_from_expected()}}
\item \href{#method-DataQuality-effective_frequency}{\code{DataQuality$effective_frequency()}}
\item \href{#method-DataQuality-get_duration}{\code{DataQuality$get_duration()}}
\item \href{#method-DataQuality-precision_using_moving_window}{\code{DataQuality$precision_using_moving_window()}}
\item \href{#method-DataQuality-clone}{\code{DataQuality$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-new"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-new}{}}}
\subsection{Method \code{new()}}{
Creates a new DataQuality object from gaze data and timestamps.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$new(gaze_x, gaze_y, timestamps, unit, screen = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{gaze_x}}{Horizontal gaze positions (pixels or degrees).}

\item{\code{gaze_y}}{Vertical gaze positions (pixels or degrees).}

\item{\code{timestamps}}{Vector of timestamps in seconds.}

\item{\code{unit}}{Unit of gaze data: either \code{"pixels"} or \code{"degrees"}.}

\item{\code{screen}}{Optional \code{ScreenConfiguration} object, required if unit is \code{"pixels"}.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new DataQuality object.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq <- DataQuality$new(gaze_x, gaze_y, timestamps, unit = "pixels", screen = sc)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-accuracy"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-accuracy}{}}}
\subsection{Method \code{accuracy()}}{
Calculates the accuracy of gaze data relative to a known target location.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$accuracy(target_azi, target_ele, central_tendency_fun = mean)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target_azi}}{Target azimuth in degrees.}

\item{\code{target_ele}}{Target elevation in degrees.}

\item{\code{central_tendency_fun}}{Function to compute central tendency (e.g., \code{mean}, \code{median}).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Accuracy in degrees.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$accuracy(0, 0)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-precision_RMS_S2S"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-precision_RMS_S2S}{}}}
\subsection{Method \code{precision_RMS_S2S()}}{
Calculates precision as root mean square of sample-to-sample distances
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$precision_RMS_S2S(central_tendency_fun = mean)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{central_tendency_fun}}{Function to compute central tendency (e.g., \code{mean}, \code{median}).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Precision in degrees.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$precision_RMS_S2S()
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-precision_STD"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-precision_STD}{}}}
\subsection{Method \code{precision_STD()}}{
Calculates precision as standard deviation of gaze positions.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$precision_STD()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Standard deviation in degrees.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$precision_STD()
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-precision_BCEA"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-precision_BCEA}{}}}
\subsection{Method \code{precision_BCEA()}}{
Calculates the Bivariate Contour Ellipse Area (BCEA) and ellipse parameters for gaze precision.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$precision_BCEA(P = 0.68)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{P}}{Proportion of data to include in the ellipse (default is 0.68).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
BCEA in degrees-squared.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$precision_BCEA()
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-data_loss_from_invalid"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-data_loss_from_invalid}{}}}
\subsection{Method \code{data_loss_from_invalid()}}{
Calculates the proportion of missing data (coded as NA).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$data_loss_from_invalid()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Proportion of missing samples.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$data_loss_from_invalid()
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-data_loss_from_expected"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-data_loss_from_expected}{}}}
\subsection{Method \code{data_loss_from_expected()}}{
Estimates data loss based on expected number of samples given the duration and sampling frequency.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$data_loss_from_expected(frequency)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{frequency}}{Expected sampling frequency in Hz.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Proportion of missing samples.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$data_loss_from_expected(500)
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-effective_frequency"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-effective_frequency}{}}}
\subsection{Method \code{effective_frequency()}}{
Calculates the effective sampling frequency based on timestamps.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$effective_frequency()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Effective frequency in Hz.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$effective_frequency()
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-get_duration"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-get_duration}{}}}
\subsection{Method \code{get_duration()}}{
Computes the total duration of the gaze recording, including the last sample.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$get_duration()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Duration in seconds.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$get_duration()
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-precision_using_moving_window"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-precision_using_moving_window}{}}}
\subsection{Method \code{precision_using_moving_window()}}{
Calculates precision using a moving window approach.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$precision_using_moving_window(
  window_length,
  metric,
  aggregation_fun = median,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{window_length}}{Length of the moving window in number of samples.}

\item{\code{metric}}{Precision metric to use (\code{"RMS-S2S"}, \code{"STD"}, or \code{"BCEA"}).}

\item{\code{aggregation_fun}}{Function to aggregate windowed precision values (e.g., \code{median}).}

\item{\code{...}}{Additional arguments passed to the precision metric function.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Precision value.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{dq$precision_using_moving_window(20, "RMS-S2S")
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-DataQuality-clone"></a>}}
\if{latex}{\out{\hypertarget{method-DataQuality-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{DataQuality$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
